package botworld;

import java.awt.*;

import prolog.*;
import prolog.math.*;

/**
 * Bars.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: Bar.java,v 1.4 2000/08/10 12:51:02 peteg Exp $
 */
public class Bar extends WorldObject {
    /** Length of a bar. */
    public static final int length = 50;

    /**
     * Angle of the bar.
     */
    double angle;

    /** Speech bubble. */
    String speech;

    /** Indicates we're being carted about. */
    boolean locked;

    /**
     * Constructs a new Bar with given centre and angle. The bounding box
     * for this object is really just the point at it's centre; nothing can
     * collide with a bar.
     */
    public Bar(int bid, int centreX, int centreY, double init_angle) {
	id = bid;
	bounds = new Rectangle(centreX, centreY, 0, 0);
	angle = init_angle;
	locked = false;
    }

    /**
     * True if the specified bot is in position (and facing the right way)
     * to pick up this bar.
     */
    public boolean onBar(Bot b) {
	return true;
    }

    /**
     * Locks this bar so that only one bot can pick it up.
     */
    public synchronized boolean lock() {
 	return locked? false : (locked = true);
    }

    /**
     * Unlocks this bar (e.g. when we get dropped).
     */
    public synchronized void unlock() {
	locked = false;
    }

    public Compound getFact() {
	Term[] args = new Term[] {
	    IntNum.make(id),
	    IntNum.make(bounds.x),
	    IntNum.make(bounds.y),
	    new FloNum(angle),
	};

	return new Compound("bar", args);
    }

    public void draw(Graphics g) {
	g.setColor(Color.green);

	int x = bounds.x,
	    y = bounds.y,
	    dx = (int)(Math.cos(angle * RADIANS) * length / 2),
	    dy = (int)(Math.sin(angle * RADIANS) * length / 2);

	g.drawLine(x - dx, y - dy, x + dx, y + dy);
    }
}
