package botworld;

import java.awt.*;

import prolog.*;

/**
 * Abstract base class of all objects in the world.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: WorldObject.java,v 1.2 2000/06/28 14:19:47 peteg Exp $
 */
public abstract class WorldObject {
    /** The rectangular bounding box of this object. */
    Rectangle bounds;

    /** The number of this object. */
    int id;

    /** Convert degrees to radians (multiplier). */
    public static final double RADIANS = Math.PI / (double)180;

    /** Convert radians to degrees (multiplier). */
    public static final double DEGREES = (double)180 / Math.PI;

    /**
     * Collision detection: This object collides with that if their bounding
     * boxes overlap.
     */
    public boolean collision(WorldObject wo) {
	return(bounds.intersects(wo.bounds));
    }

    /**
     * Build a Prolog fact to describe this object.
     */
    public abstract Compound getFact();

    /**
     * Draws this object on the provided graphics object.
     */
    public abstract void draw(Graphics g);
}
