package prolog;

/**
 * Represents an activation record in the execution of a list of goals in
 * the interpreter.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: ActivationRecord.java,v 1.7 2000/10/23 22:26:38 peteg Exp $
 */
class ActivationRecord {
    /**
     * The variable instantions from when the head of this clause was
     * unified.
     */
    Term[] insts;

    /**
     * When these goals have been solved, continue execution at this
     * <TT>ActivationRecord</TT>, using this <TT>Goal</TT>.
     */
    int cont_ar;
    Goal cont_code;

    /**
     * The cut pointer for this clause (i.e. cut back to this
     * <TT>ChoicePoint</TT>). "B0"
     */
    int cut_ptr;

    public String toString() {
	StringBuffer out = new StringBuffer();

	out.append("cont_code:\t");
	out.append(cont_code);
	out.append("\ncont_ar: ");
	out.append(cont_ar);
	out.append("\ncut_ptr: ");
	out.append(cut_ptr);
	out.append("\ninstantiations:\n");

	if(insts == null) {
	    out.append("(null)");
	} else {
	    for(int i = 0; i < insts.length; i++) {
		out.append("[");
		out.append(i);
		out.append("] = " );
		out.append(insts[i]);
		out.append("\n");
	    }
	}

	return out.toString();
    }
}
