package prolog;

/**
 * Represents a point in the execution of a Prolog program where another
 * clause could have been selected (for use on backtracking).
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: ChoicePoint.java,v 1.11 2000/08/17 13:01:22 peteg Exp $
 */
public class ChoicePoint {
    /**
     * Upon backtracking, execution at this <TT>ActivationRecord</TT>, using
     * this <TT>Goal</TT>.
     */
    int cont_ar;
    Goal cont_code;

    /**
     * The next alternative clause. "BP"
     */
    public ClauseList next_clause;

    /**
     * Top of the trail when this choicepoint was created. "TR"
     */
    Binding trail_top;

    /**
     * The <TT>ActivationRecord</TT> previous to this one with
     * alternatives. "B"
     */
    int cp_last;

    /**
     * The cut pointer for this clause (i.e. cut back to this
     * <TT>ChoicePoint</TT>. "B0"
     */
    int cut_ptr;

    /*
     * Fields specific to builtin predicates.
     */

    /* retract/1 */
    public DynamicPredicate predicate;
    public Compound arg0;

    public ChoicePoint() {
	cut_ptr = -100;
    }

    /**
     * Sets the contents of this ChoicePoint.
     */
    public final void set(int cont_ar,
			  Goal cont_code,
			  ClauseList next_clause,
			  Binding trail_top,
			  int cp_last,
			  int cut_ptr) {
	this.cont_ar     = cont_ar;
	this.cont_code   = cont_code;
	this.next_clause = next_clause;
	this.trail_top   = trail_top;
	this.cp_last     = cp_last;
	this.cut_ptr     = cut_ptr;
    }

    public String toString() {
	StringBuffer out = new StringBuffer();

	out.append("----\ncont_ar: ");
	out.append(cont_ar);
	out.append("\ncont_code:\t");
	out.append(cont_code);
 	out.append("\ncp_last: ");
 	out.append(cp_last);
 	out.append("\ncut_ptr: ");
 	out.append(cut_ptr);
 	out.append("\nnext_clause:\n");
 	out.append(next_clause);
 	out.append("\n----\n");

	return out.toString();
    }
}
