package prolog;

/**
 * Abstract base class for determinate (pre-)compiled predicates.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: CompiledPredicate.java,v 1.6 2000/08/17 13:01:22 peteg Exp $
 */
public abstract class CompiledPredicate extends Predicate {
    /**
     * Creates a new <TT>CompiledPredicate</TT>.
     */
    protected CompiledPredicate() {
    }

    public boolean unify(Term[] args, Prolog p) throws PrologException {
// 	System.err.println("CompiledPredicate.unify(): goal = " + goal);

	p.use_pending_goal();

	return exec(args, p);
    }

    /**
     * Implements the functionality of the call.
     *
     * @param args the arguments of the current goal.
     * @param p the <TT>Prolog</TT> engine, for state information.
     * @return <TT>true</TT> if the call succeeds, <TT>false</TT> otherwise.
     * @exception prolog.PrologException on e.g. typing errors.
     */
    public boolean exec(Term[] args, Prolog p) throws PrologException {
	throw new RuntimeException("CompiledPredicate.exec()");
    }

    /**
     * Default implementation of the redo method: we assume the builtin is
     * determinate, and so this should never be invoked.
     */
    public boolean redo(Term[] args, Prolog p, ChoicePoint cp)
	throws PrologException {

	throw new PrologException("CompiledPredicate.redo(): determinate builtin!");
    }

    protected Object clone() {
	return this;
    }

    public String toString() {
	return "\n(builtin)\n";
    }
}
