package prolog;

import java.io.*;

import prolog.util.Vector;

/**
 * Represents a logical connective in the body of a clause.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: Goal.java,v 1.11 2000/10/02 12:09:33 peteg Exp $
 */
public abstract class Goal implements Cloneable, Serializable {
    Goal() {
    }

    public abstract boolean unify(Term[] insts, Prolog p)
	throws PrologException;

    public abstract boolean  redo(Term[] insts, Prolog p, ChoicePoint cp)
	throws PrologException;

    /**
     * Instantiate a list of Goals with the given substitutions. We
     * overwrite the old <TT>Compound</TT> objects in the tree with their
     * instantiated counterparts.
     */
    abstract void instantiate(Term[] insts);

    /**
     * Copies and normalises the body of a predicate.
     * <P>
     * The body consists of unify()'s and calls to other
     * predicates. Ensure the following conditions hold:
     * <PRE>
     * 1. All unifications have the form: unify(X, S).
     *      where: X is a variable
     *             S is one of: variable
     *                          number
     *                          structure containing new variables.
     * 2. Calls contain variables only.
     * </PRE>
     * @see prolog.JITPredicate#normalise
     */
    abstract Goal normalise(Variable[] vars,
			    int next_free_var,
			    Vector new_vars,
			    TypeDictionary types);
    
    /**
     * @param side_effects have any calls up to now caused side effects?
     */
    boolean compile(ClauseCodeGen ccg) {
	throw new RuntimeException("compile() not implemented for: " + this);
    }

    public abstract String toString();
}
