package prolog;

import prolog.math.IntNum;

/* The JavaClass package. */
import de.fub.bytecode.classfile.*;
import de.fub.bytecode.generic.*;

/**
 * Represents the <TT>Numeric</TT> type.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: IntType.java,v 1.4 2000/11/12 12:22:45 peteg Exp $
 */
public class IntType extends NumericType {
    final static String ClassName = "prolog.math.IntNum";
    final static ObjectType JavaType = new ObjectType("prolog/math/IntNum");

    public IntNum val;

    public IntType() {
    }

    public IntType(IntNum val) {
	this.val = val;
    }

    ObjectType getJavaType(TermInfo ti) {
	return JavaType;
    }

    TermInfo instantiate(Term t, TermInfo[] typevars,
			 TypeDictionary types, boolean map) {
// 	System.err.println("IntNum.instantiate(): " + this + " against " + t);

	IntNum n = (IntNum)t;

	return (t.ti = new TermInfo(new IntType(n), typevars));

// 	throw new RuntimeException();
    }

    boolean unify(TermInfo ti,
		  TermInfo arg,
		  TypeTrail trail) {
// 	System.err.println("IntType.unify(): " + this + " against " + arg.type);

	if(arg.type instanceof TypeVariable) {
	    return arg.type.unify(arg, ti, trail);
	}

	// FIXME
	return arg.type instanceof IntType;
    }

    public boolean compile_unify(ClauseCodeGen ccg,
				 TermInfo arg0, TermInfo arg1) {
 	if(arg1.type instanceof TypeVariable) {
 	    // Reverse data flow.
 	    return arg1.type.compile_unify(ccg, arg1, arg0);
 	}

	if(!(arg1.type instanceof IntType)) {
	    throw new RuntimeException("IntType.compile_unify() -- types don't match!");
	}

	ccg.numeric_unify(arg0, arg1);

	return true;
    }

    public void create(ClauseCodeGen ccg, TermInfo arg0) {
	ccg.intnum_create(val);
    }

    public void toString(StringBuffer out) {
	out.append("IntNum");
    }
}
