package prolog;

import java.io.*;
import java.util.Enumeration;

import prolog.util.Hashtable;

/* The JavaClass package. */
import de.fub.bytecode.classfile.*;
import de.fub.bytecode.generic.*;

/**
 * Loads a class compiled by the Compiler. This is based on sample code from
 * the JavaClass API, specifically the Poor Man's Genericity project.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: JITClassLoader.java,v 1.7 2000/10/29 01:29:10 peteg Exp $
 * @see prolog.PredicateCodeGen
 */
class JITClassLoader extends ClassLoader implements Serializable {
    private Hashtable classes = new Hashtable();

    /**
     * Turn a <TT>JavaClass</TT> object into a usable Java <TT>Class</TT>
     * object.
     */
    protected final synchronized Class registerClass(JavaClass clazz)
	throws ClassNotFoundException, IOException {

	ByteArrayOutputStream bout = new ByteArrayOutputStream();
	Class c;
	String class_name;

	clazz.dump(bout);
	class_name = clazz.getClassName();
        c = defineClass(class_name, bout.toByteArray(), 0, bout.size());
	classes.put(class_name, c);

	return loadClass(class_name, true);
    }

    /**
     * Actually loads the class. Doesn't bother thinking too hard about it,
     * as this will only ever be called by the compiler.
     *
     * @exception ClassNotFoundException class not found by us or the
     *            primordial ClassLoader.
     */
    public synchronized Class loadClass(String class_name, boolean resolve)
	throws ClassNotFoundException {

//  	System.err.println("        >>>>>> loading: " + class_name);

        /* Check our local cache of classes */
        Class c = (Class)classes.get(class_name);
        if(c != null) {
//             System.out.println("        >>>>>> returning cached result: " + class_name);
        } else {
	    /* Ask the default loader for it. */
//             System.out.println("        >>>>>> using primitive ClassLoader: " + class_name);
	    c = Class.forName(class_name);
	    if(c != null) {
// 		System.out.println("        >>>>>> loaded: " + class_name);
		classes.put(class_name, c);
	    } else
		throw new ClassNotFoundException();
	}

	if(resolve) {
	    resolveClass(c);
	}

	return c;
    }

    public String toString() {
	StringBuffer out = new StringBuffer();

	out.append("--- JITClassLoader: known classes ---\n");
	for(Enumeration e = classes.keys(); e.hasMoreElements(); ) {
	    out.append(e.nextElement());
	    out.append("\n");
	}

	return out.toString();
    }
}
