package prolog;

import prolog.util.Vector;

/**
 * Representation of variables in the database (placeholders).
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: Variable.java,v 1.25 2000/11/07 12:04:56 peteg Exp $
 */
public class Variable extends Term {
    /**
     * Index of the variable referenced by this term (relative to the clause
     * it's in).
     */
    protected int id;

    public Variable(int id) {
	this.id = id;
    }

    public Term instantiate(Term[] insts) {
	if(insts[id] == null) {
	    return insts[id] = new Binding(id);
	} else {
	    return insts[id];
	}
    }

    public boolean unify(Term t, Term[] insts, Prolog p) {
	if(insts[id] == null) {
	    /* We're an arg var, so unify with anything. */
	    insts[id] = t;
// 	    insts[id] = t.getBinding();
	    return true;
	} else {
	    /* We're already bound; let our binding do the job. */
	    return insts[id].unify(t, insts, p);
	}
    }

    public boolean identical(Term t) {
	throw new RuntimeException("Variable.identical() called.");
    }

    public Term normalise_copy(Variable[] vars,
			       int next_new_var,
			       Vector new_vars) {
	if(vars[id] == null) {
	    // Allocate a new variable.
	    Variable newVar = new Variable(next_new_var);

	    new_vars.addElement(vars[id] = newVar);
	    return newVar;
	} else {
	    return vars[id];
	}
    }

    public void typify(TypeDictionary types, boolean map) {
	if(ti == null) {
// 	    System.err.println("Variable.typeify(): " + this);
	    ti = new TermInfo(new TypeVariable(id, false));
	}
    }

    int getID() {
	return id;
    }

    void setID(int id) {
	this.id = id;
    }

    public String getName() {
	return "_" + id;
    }

    public String displayString() {
	return toString();
    }

    public String toString() {
	char c = (char)('A' + (id % 26));
	String s = "_" + c + (id / 26);

	return s;
    }
}
