package prolog.lib;

import prolog.*;

/**
 * <TT>_type_def/2</TT>: Introduce a new type.
 * <P>
 * <TT>Type ::= Component1, Component2, ...</TT>
 * <P>
 * where each <TT>Component</TT> is a disjunctive component of this
 * <TT>Type</TT>.
 * <P>
 * See "A Polymorphic Type System for Prolog" - Mycroft and O'Keefe,
 * Artificial Intelligence 23 (1984) 295-307.
 * <P>
 * The syntax of <TT>Component</TT>s is as follows:
 * <P>
 * <PRE>
 * Component -> Constructor Types
 *
 * Types     -> (empty)
 *           -> "(" Types' ")"
 * Types'    -> Type
 *           -> Types' "," Type
 *
 * Type      -> "Numeric" | "Binding" | Variable
 * </PRE>
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: _type_def_2.java,v 1.3 2000/09/03 12:48:04 peteg Exp $
 * @see prolog.TypeDictionary
 * @see prolog.lib._has_type_2
 * @see prolog.lib.listtypes_0
 */
public class _type_def_2 extends CompiledPredicate {
    public _type_def_2() {
    }

    public boolean exec(Term[] args, Prolog p) throws ArgumentException {
	try {
	    p.types.type_def((Compound)args[0], (Compound)args[1]);

	    return true;
	} catch(ClassCastException e) {
	    throw new ArgumentException("::=/2: invalid arguments: ", args);
	}
    }
}
