package prolog.lib;

import prolog.*;

/**
 * <TT>gensym/2</TT>: Create unique versions of a symbol.
 * <P>
 * Usage: <TT>gensym(f, F)</TT> binds F to f<I>x</I>, where <I>x</I> is a
 * new, monotonically increasing, number (with respect to previous calls).
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: gensym_2.java,v 1.6 2000/09/03 12:48:05 peteg Exp $
 */
public class gensym_2 extends CompiledPredicate {
    /**
     * The counter.
     */
    private int counter = 0;

    public gensym_2() {
    }

    public boolean exec(Term[] args, Prolog p) throws ArgumentException {
	try {
	    Compound arg0 = (Compound)args[0].getBinding();
	    Term     arg1 = args[1].getBinding();

//   	    System.err.println("gt: " + arg0.val.toString() + " > " + arg1.val.toString());

	    return arg1.unify(new Compound(arg0.functor + counter++), null, p);
	} catch(ClassCastException e) {
	    throw new ArgumentException("gensym/2: invalid arguments: ", args);
	}
    }
}
