package prolog.lib;

import prolog.*;
import prolog.math.Numeric;

/**
 * <TT>is_equal/2</TT>: The first argument evaluates to the same number as
 * the second (when both are interpreted as arithmetical expressions).
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: is_equal_2.java,v 1.6 2000/08/10 12:51:04 peteg Exp $
 * @see prolog.lib.is_2
 * @see prolog.lib.is_not_equal_2
 */
public class is_equal_2 extends CompiledPredicate {
    public is_equal_2() {
    }

    public boolean exec(Term[] args, Prolog p) throws ArgumentException {
	try {
//  	    System.err.println("is_equal/2");

	    Term arg0 = args[0].getBinding(), arg1 = args[1].getBinding();
	    Numeric r0, r1;

	    if(arg0 instanceof Compound) {
		r0 = is_2.eval((Compound)arg0);
	    } else {
		r0 = (Numeric)arg0;
	    }

	    if(arg1 instanceof Compound) {
		r1 = is_2.eval((Compound)arg1);
	    } else {
		r1 = (Numeric)arg1;
	    }

	    return r0.equals(r1);
	} catch(ClassCastException e) {
	    throw new ArgumentException("is_equal/2: invalid arguments: ", args);
	}
    }
}
