package prolog.math;

import prolog.*;
import prolog.util.Vector;

/**
 * Maths package for Prolog. A stripped and modified version of the gnu.math
 * library, which has the following copyright:
 * <P>
 * Copyright (c) 1997  Per M.A. Bothner.
 * <B>This is free software;  for terms and warranty disclaimer see ./COPYING.</B>
 * <P>
 * FIXME: This is the floating-point class. (double precision).
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: FloNum.java,v 1.8 2000/11/03 11:08:11 peteg Exp $
 */
public class FloNum extends Numeric {
    double value;

    public FloNum(double value) {
	this.value = value;
    }

    public Term normalise_copy(Variable[] vars,
			       int next_new_var,
			       Vector new_vars) {
	return new FloNum(value);
    }

    public void typify(TypeDictionary types, boolean map) {
	ti = new TermInfo(new FloType(this));

	System.err.println(">> FloNum.typeify(): " + this
			   + " -> .ti: " + ti);
    }

    public final double doubleValue() {
	return value;
    }

    public long longValue() {
	return (long)value;
    }

    public int hashCode() {
	return (int)value;
    }

    public boolean equals (Object obj) {
	// take from java.lang.Double.equals:
	return (obj != null)
	    && (obj instanceof FloNum) 
	    && (((FloNum)obj).value == value);
    }

    public Numeric add(Numeric y) {
	return new FloNum(value + y.doubleValue());
    }

    public Numeric sub(Numeric y) {
	return new FloNum(value - y.doubleValue());
    }

    public Numeric mul (Numeric y) {
      return new FloNum(value * y.doubleValue());
    }

    public Numeric div(Numeric y) {
	return new FloNum (value / y.doubleValue());
    }

    public Numeric mod(Numeric y) {
	return new FloNum(value % y.doubleValue());
    }

    public Numeric power(IntNum y) {
	return new FloNum(Math.pow(doubleValue(), y.doubleValue()));
    }

    public boolean isNegative() {
	return value < 0;
    }

  public Numeric neg ()
  {
    return new FloNum (-value);
  }

  public int sign ()
  {
    return value > 0.0 ? 1 : value < 0.0 ? -1 : value == 0.0 ? 0: -2;
  }

  public static int compare (double x, double y)
  {
    return x > y ? 1 : x < y ? -1 : x == y ? 0 : -2;
  }

    public int compare(Numeric n) {
	return compare(value, n.doubleValue());
    }

  public boolean isZero ()
  {
    return value == 0.0;
  }


    public String toString () {
	return(value == 1.0/0.0 ? "#i1/0"
	       : value == -1.0/0.0 ? "#i-1/0"
	       : Double.isNaN (value) ? "#i0/0"
	       : Double.toString (value));
    }

    public String toString(int radix) {
	if(radix == 10) {
	    return toString();
	} else {
	    return "#d" + toString();
	}
    }
}
