package prolog.util;

import prolog.*;

/**
 * Library of string manipulation functions.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: Strings.java,v 1.4 2000/09/15 15:11:35 peteg Exp $
 */
public final class Strings {
    /**
     * The table that maps escapes -> characters
     */
    private static final char[] escapes = {'\"', '\'', '\\', 'b', 'f', 'n', 'r', 't'};
    private static final char[] escape_map = {'\"', '\'', '\\', '\b', '\f', '\n', '\r', '\t'};

    private Strings() {
    }

    /**
     * Converts escaped characters into their unescaped form.
     *
     * @exception prolog.ParsingException thrown if the string contains an
     * unknown escape sequence.
     */
    public static String expand_escapes(String s) throws ParsingException {
	if(s == null) {
	    return null;
	}

	StringBuffer sb = new StringBuffer();

	for(int i = 0; i < s.length(); i++) {
	    char ch = s.charAt(i);

	    if(ch == '\\' && i < s.length() - 1) {
		ch = s.charAt(++i);

		/* Linear search the escapes list. */
		int j;

		for(j = 0; j < escapes.length && escapes[j] != ch; j++);
		if(j >= escapes.length) {
		    throw new ParsingException("prolog.util.Strings: invalid escape in \"" + s + "\"");
		}

		sb.append(escape_map[j]);
	    } else {
		sb.append(ch);
	    }
	}

	return sb.toString().intern();
    }

    /**
     * Escapes unprintable characters as necessary.
     */
    public static String escape_string(String s) {
	if(s == null) {
	    return null;
	}

	StringBuffer sb = new StringBuffer();

	for(int i = 0; i < s.length(); i++) {
	    char ch = s.charAt(i);

	    /* Check for character to be escaped... */
	    for(int j = 0; j < escape_map.length; j++) {
		if(ch == escape_map[j]) {
		    sb.append('\\');
		    ch = escapes[j];
		    break;
		}
	    }

	    /* ...otherwise add it verbatim. */
	    sb.append(ch);
	}

	return sb.toString().intern();
    }
}
