package prolog.lib;

import prolog.*;

/**
 * member/2: Membership of a list. Implemented to test support for
 * backtracking through compiled predicates.
 *
 * <PRE>
 * member(X, [X|_]).
 * member(X, [_|Ys]) :- member(X, Ys).
 * </PRE>
 *
 * Normalised:
 * <PRE>
 * member(X, Y) :-
 *	unify(Y, .(A, B)),
 *	unify(X, A).
 * member(X, Y) :-
 *	unify(Y, .(A, B)),
 *	member(X, B).
 * </PRE>
 *
 * <B>Note:</B> only works in the mode <TT>member(+, +)</TT> i.e. both args
 * bound - second is a list. Also, everything is ground.
 *
 * @author Peter Gammie
 */
public class member_2 extends CompiledClause {
    public member_2() {
	super(true);
    }

    public boolean unify(Term[] args, Prolog s) {
	ChoicePoint cp = s.cp;

	if(cp.args != null) {
	    args = cp.args;
	} else {
	    cp.args = args;
	}

	Term arg0 = args[0];
	Compound arg1 = (Compound)args[1];

	while(true) {
	    switch(cp.clauseNum) {
	    case 1:
//  		System.err.println("member/2(1): arg1 = " + arg1);

		if(arg1 instanceof List) {
		    if(arg0.unify(arg1.args[0], null, s)) {
			cp.clauseNum = 2;
			args[1] = arg1;
			return(true);
		    }
		}

	    case 2:
//  		System.err.println("member/2(2): arg1 = " + arg1);

		if(arg1 instanceof List) {
		    arg1 = (Compound)arg1.args[1];
		    cp.clauseNum = 1;
		    continue;
		}

	    case 3:
		cp.finished();
		return(false);
	    }
	}
    }
}
