package prolog.pred;

import prolog.*;

/**
 * member/2: Membership of a list. Implemented to test support for
 * backtracking through compiled predicates.
 * <PRE>
 * member(X, [X|_]).
 * member(X, [_|Ys]) :- member(X, Ys).
 * </PRE>
 * @author Peter Gammie
 */

public class member_2 extends Pred {
    public member_2() {
	functor = "member";
	arity = 2;
	usecp = true;
    }

    public boolean unify(Term t, Prolog s) {
	Compound st = (Compound)t;
	Term[] tmp_args;

	// Grab a copy of the  arguments if we haven't already
	if(st.tmp == null) {
// 	    System.err.println("Copying args -- " + st.args[0] + " -- " + st.args[1]);
	    st.tmp = tmp_args = new Term[st.arity];
	    for(int i = 0; i < st.arity; i++)
		tmp_args[i] = st.args[i];
	} else
	    tmp_args = (Term[])st.tmp;

	Term arg0 = tmp_args[0];
	Term arg1 = tmp_args[1];
	int clause = s.cp.clausenum;

	while(true) {
	    switch(clause) {
	    // member(Arg0, Arg1) :- unify(Arg1, [Arg0|Xs]).
	    case 0: {
// 		System.err.println("member - clause0");

		Variable xs = new Variable(null, 0);
		Compound arg0xs = new Compound(".", new Term[]{arg0, xs});

// 		System.err.println("arg0 = " + arg0);
// 		System.err.println("arg1 = " + arg1);
// 		System.err.println("arg0Xs = " + arg0xs);

		if(arg0xs.unify(arg1, s)) {
// 		    System.err.println("[Arg0|Xs] binding: " + arg1);
		    s.cp.clausenum = 1;
		    return(true);
		}
		s.backtrack();
	    }
	    // member(Arg0, Arg1) :- unify(Arg1, [Y|Ys]), member(Arg0, Ys).
	    case(1): {
// 		System.err.println("member - clause1");

		Variable y = new Variable(null, 0);
		Variable ys = new Variable(null, 1);
		Compound yys = new Compound(".", new Term[]{y, ys});

// 		System.err.println("arg0 = " + arg0);
// 		System.err.println("arg1 = " + arg1);
// 		System.err.println("yys = " + yys);

		if(yys.unify(arg1, s)) {
// 		    System.err.println("[Y|Ys] binding: " + yys);
 		    tmp_args[1] = arg1 = ys;
		    clause = 0;
 		    s.cp.bindingIndex = s.bindingStack;
		    continue;
		}
		s.backtrack();
	    }
	    default: {
		s.cp.finished();
		st.tmp = null; // Ditch the copied args
		return(false);
	    }
	    }
	}
    }
}
