% -*- Prolog -*-
%dynamic(cycles/1, weights/1, done/0)!


%inputList( [[0, 1, 1, 0, 0],
%			[1, 0, 0, 1, 0],
%			[1, 0, 0, 0, 0],
%			[0, 1, 1, 1, 1],
%			[0, 0, 1, 0, 0],
%			[0, 0, 0, 1, 0],
%			[0, 0, 1, 1, 0],
%			[1, 0, 1, 0, 0],
%			[1, 1, 1, 1, 0],
%			[1, 0, 1, 1, 1],
%			[1, 1, 0, 0, 0],
%			[1, 1, 0, 1, 1],
%			[0, 0, 0, 0, 0],
%			[0, 1, 0, 1, 0],
%			[1, 1, 1, 0, 1],
%			[0, 1, 0, 0, 0]]).

inputList( [[1.2, 1, 0],
			[1.2, 0, 0],
			[0.1, 0, 0],
			[0.9, 1, 1], 
			[0.1, 0, 0],
			[2, 0, 0],
			[-0.9, -1, 0],
			[1, 0, 0],
			[1.05, 1, 0], 
			[1, 1.1, 1],
			[1.1, 1, 0],
			[1, 1.1, 1],
			[0.1, 0, 0],
			[1.2, 0, 0],
			[1, 1.05, 1], 
			[2, 1, 0]]).

lrate(0.1).

%%
%%

last([X], X).
last([H|T], X) :- last(T, X).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  generate a list of 'Size' elements -- set to random values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

weightList([X], 0) :- !, X is random(0.1).   % the bias weight

weightList([Head|Tail], Size) :-
	is(Head, random(0.1)),
	is(Newsize, Size - 1),
	weightList(Tail, Newsize).

%%%%%%

calculateInput([X], [BiasWeight], Total, Result) :-
	Result is Total + BiasWeight.

calculateInput([Input, B| Inputs], [Weight | Weights], Total, Result) :-
	NewTotal is Total + Input * Weight,
	calculateInput([B|Inputs], Weights, NewTotal, Result).
	 

calculateOutput(Input, 0) :- le(Input, 0).
calculateOutput(Input, 1) :- gt(Input, 0).

%%%%%%%

run(MaxCycles) :-
	inputList([H|T]),
	length(H, Len),
	NumInputs is Len - 1,    % because one is the classification, not an input
	weightList(Weights, NumInputs),   % generate random weights
	train([H|T], Weights, 0, MaxCycles).
	
run(_) :-
	repeat, 
	inputList(L), 
	weights(W), 
	%%%%%print(W),
	cycles(C), 
	train(L, W, 0, C).
	

train(_, _, _, _) :-
	done, retract(done), abolish(weights/1), abolish(cycles/1),
	!, print("press ^C to exit\n"), repeat, ratom(X).

train(_, W, _, 0) :- 
	!, print("\ntraining time elapsed\n"), print(W), assert(done).    % training time has elapsed

train(_, W, NumIn, _) :-   % sucessfully learnt
	inputList(L), length(L, NumIn), !, print("\nlearning completed\n"), print(W), assert(done).   

train([], A, B, C) :- !, abolish(weights/1), abolish(cycles/1), 
						assert(weights(A)), assert(cycles(C)).


train([Input|Inputs], Weights, Correct, CyclesLeft) :-
	calculateInput(Input, Weights, 0, Value),
	calculateOutput(Value, Output),
	last(Input, Desired),
	Output = Desired,
	NewCorrect is Correct + 1,
	NewCycles is CyclesLeft - 1,
	train(Inputs, Weights, NewCorrect, NewCycles).
	

%% perceptron generated incorrect classifier, so we adjust weights

train([Input|Inputs], Weights, Correct, CyclesLeft) :-
	calculateInput(Input, Weights, 0, Value),   % redundant
	calculateOutput(Value, Output),          % redundant
	last(Input, Desired),					 % redundant
	not(Output = Desired), 				 % redundant
	Error is Desired - Output, 
	adjustWeights(Weights, NewWeights, Input, Error),
	NewCycles is CyclesLeft - 1,
	train(Inputs, NewWeights, 0, NewCycles).



adjustWeights([Weight], [NewWeight], [Class], Error) :-
	lrate(Lrate),
	NewWeight is Weight + Lrate * Error.     % the bias input

adjustWeights([Weight | Weights], [NewWeight | OtherNew], [Input | Inputs], Error) :-
	lrate(Lrate),
	NewWeight is Weight + Lrate * Error * Input,
	adjustWeights(Weights, OtherNew, Inputs, Error).
	
	 	
	



