%dynamic(cycles/1, weights/1, done/0)!

/*
inputList( [[0, 1, 1, 0, 0],
			[1, 0, 0, 1, 0],
			[1, 0, 0, 0, 0],
			[0, 1, 1, 1, 1], 
			[0, 0, 1, 0, 0],
			[0, 0, 0, 1, 0],
			[0, 0, 1, 1, 0],
			[1, 0, 1, 0, 0],
			[1, 1, 1, 1, 0], 
			[1, 0, 1, 1, 1],
			[1, 1, 0, 0, 0],
			[1, 1, 0, 1, 1],
			[0, 0, 0, 0, 0],
			[0, 1, 0, 1, 0],
			[1, 1, 1, 0, 1], 
			[0, 1, 0, 0, 0]]).
*/

inputList( [[1.2, 1, 0],
			[1.2, 0, 0],
			[0.1, 0, 0],
			[0.9, 1, 1], 
			[0.1, 0, 0],
			[2, 0, 0],
			[-0.9, -1, 0],
			[1, 0, 0],
			[1.05, 1, 0], 
			[1, 1.1, 1],
			[1.1, 1, 0],
			[1, 1.1, 1],
			[0.1, 0, 0],
			[1.2, 0, 0],
			[1, 1.05, 1], 
			[2, 1, 0]]).



lrate(0.1).

%%
%%

last([X], X).
last([H|T], X) :- last(T, X).


randweights([0.05, 0.01, 0.072]).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  generate a list of 'Size' elements -- set to random values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

weightList(X, _) :-
	randweights(X).

/*
weightList([X], 0) :- !, X is random(0.1).   % the bias weight

weightList([Head|Tail], Size) :-
	Head is random(0.1),
	Newsize is Size - 1,
	weightList(Tail, Newsize).
*/	
	
%%%%%%

calculateInput([X], [BiasWeight], Total, Result) :-
	is(Result, Total + BiasWeight).

calculateInput([Input, B| Inputs], [Weight | Weights], Total, Result) :-
	is(NewTotal, Total + Input * Weight),
	calculateInput([B|Inputs], Weights, NewTotal, Result).
	 

calculateOutput(Input, 0) :- le(Input, 0).
calculateOutput(Input, 1) :- gt(Input, 0).

%%%%%%%

	
run(MaxCycles) :-
	inputList([H|T]),
	length(H, Len),
	is(NumInputs, Len - 1),    % because one is the classification, not an input
	weightList(Weights, NumInputs),   % generate random weights
	train([H|T], Weights, 0, MaxCycles).
	
run(_) :-
	repeat, 
	inputList(L), 
	weights(W), 
        %%%%%print(W),
        cycles(C), 
	train(L, W, 0, C).
	

train(_, _, _, _) :-
	done, retract(done), abolish(weights, 1), abolish(cycles, 1),
	!, write("press ^C to exit\n"), repeat, ratom(X).

train(_, W, _, 0) :- 
	!, write("\ntraining time elapsed\n"), write(W), assert(done).    % training time has elapsed

train(_, W, NumIn, E) :-   % sucessfully learnt
	inputList(L), length(L, NumIn), !, write("\nlearning completed\n"), write(W), write(E), assert(done).   

train([], A, B, C) :- !, abolish(weights, 1), abolish(cycles, 1), 
						assert(weights(A)), assert(cycles(C)).


train([Input|Inputs], Weights, Correct, CyclesLeft) :-
	calculateInput(Input, Weights, 0, Value),
	calculateOutput(Value, Output),
	last(Input, Desired),
	unify(Output, Desired),
	is(NewCorrect, Correct + 1),
	is(NewCycles, CyclesLeft - 1),
	train(Inputs, Weights, NewCorrect, NewCycles).
	

%% perceptron generated incorrect classifier, so we adjust weights

train([Input|Inputs], Weights, Correct, CyclesLeft) :-
	calculateInput(Input, Weights, 0, Value),   % redundant
	calculateOutput(Value, Output),          % redundant
	last(Input, Desired),					 % redundant
	not(unify(Output, Desired)), 				 % redundant
	is(Error, Desired - Output), 
	adjustWeights(Weights, NewWeights, Input, Error),
	is(NewCycles, CyclesLeft - 1),
	train(Inputs, NewWeights, 0, NewCycles).



adjustWeights([Weight], [NewWeight], [Class], Error) :-
	lrate(Lrate),
	is(NewWeight, Weight + Lrate * Error).     % the bias input

adjustWeights([Weight | Weights], [NewWeight | OtherNew], [Input | Inputs], Error) :-
	lrate(Lrate),
	is(NewWeight, Weight + Lrate * Error * Input),
	adjustWeights(Weights, OtherNew, Inputs, Error).

%%%%%%%%%%%%

unify(X, X).

not(X) :- call(X), !, fail.
not(X).

length([], 0).
length([_|Xs], L) :-
	length(Xs, L0),
	is(L, L0 + 1).

%%%%%%%%%%%

main :-
	run(10000).
